@@ -42,7 +42,7 @@
 #include "gs-theme-window.h"
 
 #ifndef trunc
-#define trunc(x) ((x > 0.0) ? floor(x) : -floor(-x))
+#define trunc(x) (((x) > 0.0) ? floor((x)) : -floor(-(x)))
 #endif
 
 #ifndef OPTIMAL_FRAME_RATE
